"""Tests for the views of the ``feedback_form`` app."""
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from django_libs.tests.factories import UserFactory
from django_libs.tests.mixins import ViewTestMixin

from ..models import Feedback


class FeedbackCreateViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``FeedbackCreateView`` generic view."""
    def setUp(self):
        self.user = UserFactory()

    def get_view_name(self):
        return 'feedback_form'

    def test_view(self):
        self.should_be_callable_when_anonymous()
        self.should_be_callable_when_authenticated(self.user)
        data = {'feedback-message': 'Foo'}
        self.is_callable(method='post', data=data)
        self.assertEqual(Feedback.objects.all().count(), 1)
        self.assertEqual(Feedback.objects.all()[0].message, 'Foo')
        self.assertEqual(Feedback.objects.all()[0].current_url, '/feedback/')

        # Test AJAX
        self.is_callable(method='post', data=data, ajax=True)
        self.is_callable(ajax=True)


class FeedbackCreateViewContentObjectTestCase(ViewTestMixin, TestCase):
    """
    Tests for the ``FeedbackCreateView`` generic view, if using a content
    object.

    """
    def setUp(self):
        self.user = UserFactory()
        self.content_object = UserFactory()

    def get_view_name(self):
        return 'feedback_form_content_object'

    def get_view_kwargs(self):
        return {
            'c_type': ContentType.objects.get_for_model(
                self.content_object).model,
            'obj_id': self.content_object.pk,
        }

    def test_view(self):
        self.should_be_callable_when_anonymous()
        self.should_be_callable_when_authenticated(self.user)
        self.is_not_callable(kwargs={'c_type': 'foo', 'obj_id': '999'})
        self.is_not_callable(kwargs={
            'c_type': ContentType.objects.get_for_model(
                self.content_object).model,
            'obj_id': '999',
        })
        data = {'feedback-message': 'Foo'}
        self.is_callable(method='post', data=data)
        self.assertEqual(Feedback.objects.all().count(), 1)
        self.assertEqual(Feedback.objects.all()[0].message, 'Foo')
        self.assertEqual(
            Feedback.objects.all()[0].content_object, self.content_object)



"""Views for the ``feedback_form`` app."""
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.http import Http404
from django.views.generic import CreateView

from .app_settings import *  # NOQA
from .forms import FeedbackForm
from .models import Feedback


class FeedbackCreateView(CreateView):
    """View to display and handle a feedback create form."""
    model = Feedback
    form_class = FeedbackForm
    ajax_template = 'feedback_form/partials/form_content.html'

    def dispatch(self, request, *args, **kwargs):
        if kwargs.get('c_type') and kwargs.get('obj_id'):
            try:
                content_type = ContentType.objects.get(model=kwargs['c_type'])
            except ContentType.DoesNotExist:
                raise Http404
            try:
                self.content_object = content_type.get_object_for_this_type(
                    pk=kwargs['obj_id'])
            except ObjectDoesNotExist:
                raise Http404
        return super(FeedbackCreateView, self).dispatch(
            request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(FeedbackCreateView, self).get_form_kwargs()
        if self.request.user.is_authenticated():
            kwargs.update({'user': self.request.user})
        kwargs.update({
            'url': self.request.META.get('HTTP_REFERER', self.request.path),
            'content_object': self.content_object if hasattr(
                self, 'content_object') else None,
        })
        return kwargs

    def get_template_names(self):
        if self.request.is_ajax():
            return self.ajax_template
        return super(FeedbackCreateView, self).get_template_names()

    def get_context_data(self, **kwargs):
        context = super(FeedbackCreateView, self).get_context_data(**kwargs)
        context.update({
            'background_color': FEEDBACK_FORM_COLOR,
            'text_color': FEEDBACK_FORM_TEXTCOLOR,
            'text': FEEDBACK_FORM_TEXT,
        })
        if hasattr(self, 'content_object'):
            context.update({'content_object': self.content_object})
        return context

    def get_success_url(self):
        return reverse('feedback_form')



# flake8: noqa
"""Settings that need to be set in order to run the tests."""
import logging

from base_settings import *
from local_settings import *


logging.getLogger("factory").setLevel(logging.WARN)



"""Settings of the ``feedback_form``` application."""
from django.conf import settings


FEEDBACK_FORM_COLOR = getattr(settings, 'FEEDBACK_FORM_COLOR', '#6caec9')
FEEDBACK_FORM_TEXTCOLOR = getattr(settings, 'FEEDBACK_FORM_TEXTCOLOR', '#fff')
FEEDBACK_FORM_TEXT = getattr(settings, 'FEEDBACK_FORM_TEXT', '')



"""URLs for the ``feedback_form`` app."""
from django.conf.urls import patterns, url

from .views import FeedbackCreateView


urlpatterns = patterns(
    '',
    url(
        r'^(?P<c_type>[-\w]+)/(?P<obj_id>\d+)/$',
        FeedbackCreateView.as_view(),
        name='feedback_form_content_object'),
    url(r'^$', FeedbackCreateView.as_view(), name='feedback_form'),
)



"""Models for the ``feedback_form`` app."""
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Feedback(models.Model):
    """
    Holds information about one user feedback.

    :user: User account of the poster, if logged in.
    :email: Email field, if user isn't logged in and wants to send her email.
    :current_url: URL of the current page.
    :message: Feedback text.
    :creation_date: Datetime of the feedback creation.
    :content_object: Optional related object the feedback is referring to.

    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        related_name='feedback_form_submissions',
        blank=True, null=True,
    )

    email = models.EmailField(
        verbose_name=_('Email'),
        blank=True,
    )

    current_url = models.URLField(
        verbose_name=_('Current URL'),
        max_length=4000,
        blank=True,
    )

    message = models.TextField(
        verbose_name=_('Message'),
        max_length=4000,
    )

    creation_date = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_('Creation Date'),
    )

    # Generic FK to the object this feedback is about
    content_type = models.ForeignKey(
        ContentType,
        related_name='feedback_content_objects',
        null=True, blank=True,
    )
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    class Meta:
        ordering = ['-creation_date']

    def __unicode__(self):
        if self.user:
            return '{0} - {1}'.format(self.creation_date, self.user)
        elif self.email:
            return '{0} - {1}'.format(self.creation_date, self.email)
        return '{0}'.format(self.creation_date)



"""Tests for the template tags and filters of the ``feedback_form`` app."""
from django.contrib.auth.models import AnonymousUser
from django.template import Template, RequestContext
from django.test import TestCase, RequestFactory

from django_libs.tests.factories import UserFactory


class FeedbackFormTestCase(TestCase):
    """Tests for the ``feedback_form`` tag."""
    longMessage = True

    def test_render_tag(self):
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        c = RequestContext(request)
        t = Template('{% load feedback_tags %}{% feedback_form %}')
        self.assertIn('<input type="submit" value="Send Feedback" />',
                      t.render(c))
        # Should add the email field, because user is anonymous
        self.assertIn('email', t.render(c))

        # Test with logged in user
        request.user = UserFactory()
        c = RequestContext(request)
        self.assertIn('<input type="submit" value="Send Feedback" />',
                      t.render(c))
        self.assertNotIn('email', t.render(c))



"""Template tags and filters for the ``feedback_form`` app."""
from django import template

from ..app_settings import *  # NOQA
from ..forms import FeedbackForm

register = template.Library()


@register.inclusion_tag('feedback_form/partials/form.html', takes_context=True)
def feedback_form(context):
    """Template tag to render a feedback form."""
    user = None
    url = None
    if context.get('request'):
        url = context['request'].path
        if context['request'].user.is_authenticated():
            user = context['request'].user
    return {
        'form': FeedbackForm(url=url, user=user),
        'background_color': FEEDBACK_FORM_COLOR,
        'text_color': FEEDBACK_FORM_TEXTCOLOR,
        'text': FEEDBACK_FORM_TEXT,
    }



"""Tests for the models of the ``feedback_form`` app."""
from django.test import TestCase

from ..models import Feedback


class FeedbackTestCase(TestCase):
    """Tests for the ``Feedback`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test if the ``Feedback`` model instantiates."""
        feedback = Feedback()
        self.assertTrue(feedback, msg='Should be correctly instantiated.')



"""Forms for the ``feedback_form`` app."""
from django import forms
from django.conf import settings
from django.core.urlresolvers import reverse

from django_libs.utils_email import send_email

from .models import Feedback


class FeedbackForm(forms.ModelForm):
    """
    A feedback form with modern spam protection.

    :url: Field to trap spam bots.

    """
    url = forms.URLField(required=False)

    def __init__(self, user=None, url=None, prefix='feedback',
                 content_object=None, *args, **kwargs):
        self.content_object = content_object
        super(FeedbackForm, self).__init__(prefix='feedback', *args, **kwargs)
        if url:
            self.instance.current_url = url
        if user:
            self.instance.user = user
            del self.fields['email']
        else:
            self.fields['email'].required = True

    def save(self):
        if not self.cleaned_data.get('url'):
            self.instance.content_object = self.content_object
            obj = super(FeedbackForm, self).save()
            send_email(
                '',
                {
                    'url': reverse('admin:feedback_form_feedback_change',
                                   args=(obj.id, )),
                    'feedback': obj,
                },
                'feedback_form/email/subject.html',
                'feedback_form/email/body.html',
                from_email=settings.FROM_EMAIL,
                recipients=[manager[1] for manager in settings.MANAGERS],
            )
            if getattr(settings, 'FEEDBACK_EMAIL_CONFIRMATION', False):
                email = None
                if obj.email:
                    email = obj.email
                elif obj.user.email:
                    email = obj.user.email
                if email:
                    send_email(
                        '', {},
                        'feedback_form/email/confirmation_subject.html',
                        'feedback_form/email/confirmation_body.html',
                        from_email=settings.FROM_EMAIL,
                        recipients=[email],
                    )
            return obj

    class Media:
        css = {'all': ('feedback_form/css/feedback_form.css'), }
        js = ('feedback_form/js/feedback_form.js', )

    class Meta:
        model = Feedback
        fields = ('email', 'message')



"""Admin objects for the ``feedback_form`` app."""
from django.contrib import admin
from django.template.defaultfilters import truncatewords
from django.utils.translation import ugettext_lazy as _

from .models import Feedback


class FeedbackAdmin(admin.ModelAdmin):
    """Admin class for the ``Feedback`` model."""
    list_display = [
        'creation_date', 'user_email', 'current_url', 'message_excerpt', ]
    list_filter = ['creation_date', 'current_url', ]
    date_hierarchy = 'creation_date'
    search_fields = ['user__email', 'email', 'current_url', 'message', ]

    def message_excerpt(self, obj):
        return truncatewords(obj.message, 10)
    message_excerpt.short_description = _('Message excerpt')

    def user_email(self, obj):
        if obj.user:
            return obj.user.email
        return obj.email
    user_email.short_description = _('Email')


admin.site.register(Feedback, FeedbackAdmin)



"""Tests for the forms of the ``feedback_form`` app."""
from django.test import TestCase

from django_libs.tests.factories import UserFactory
from mailer.models import Message

from ..forms import FeedbackForm
from ..models import Feedback


class FeedbackFormTestCase(TestCase):
    """Test for the ``FeedbackForm`` form class."""
    longMessage = True

    def test_form(self):
        data = {
            'feedback-email': 'test@example.com',
            'feedback-message': 'Foo',
            'feedback-url': 'http://www.example.com',
        }
        form = FeedbackForm(data=data)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEqual(Message.objects.all().count(), 0)
        self.assertEqual(Feedback.objects.all().count(), 0)

        with self.settings(FEEDBACK_EMAIL_CONFIRMATION=True):
            # Valid post
            data.update({'feedback-url': ''})
            form = FeedbackForm(data=data)
            self.assertTrue(form.is_valid())
            form.save()
            self.assertEqual(Message.objects.all().count(), 2)
            self.assertEqual(Feedback.objects.all().count(), 1)
            self.assertEqual(Feedback.objects.all()[0].message, 'Foo')
            self.assertEqual(Feedback.objects.all()[0].email,
                             'test@example.com')
            Feedback.objects.all()[0].delete()
            Message.objects.all()[0].delete()

            # Valid post with user account
            user = UserFactory()
            data.update({'feedback-email': ''})
            form = FeedbackForm(data=data, user=user)
            self.assertTrue(form.is_valid())
            form.save()
            self.assertEqual(Message.objects.all().count(), 3)
            self.assertEqual(Feedback.objects.all().count(), 1)
            self.assertEqual(Feedback.objects.all()[0].message, 'Foo')
            self.assertEqual(Feedback.objects.all()[0].user, user)
